@@ -24,11 +24,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-animation.h"
+#include "gdk-pixbuf-scaled-anim.h"
 #include "gdk-pixbuf-io.h"
 #include "gdk-pixbuf-loader.h"
 #include "gdk-pixbuf-marshal.h"
@@ -90,7 +91,7 @@ gdk_pixbuf_loader_class_init (GdkPixbufLoaderClass *class)
          * the desired size to which the image should be scaled.
          */
         pixbuf_loader_signals[SIZE_PREPARED] =
-                g_signal_new ("size_prepared",
+                g_signal_new ("size-prepared",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdkPixbufLoaderClass, size_prepared),
@@ -110,7 +111,7 @@ gdk_pixbuf_loader_class_init (GdkPixbufLoaderClass *class)
          * the partially-loaded pixbuf.
          */
         pixbuf_loader_signals[AREA_PREPARED] =
-                g_signal_new ("area_prepared",
+                g_signal_new ("area-prepared",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdkPixbufLoaderClass, area_prepared),
@@ -133,7 +134,7 @@ gdk_pixbuf_loader_class_init (GdkPixbufLoaderClass *class)
          * areas of an image that is being loaded.
          */
         pixbuf_loader_signals[AREA_UPDATED] =
-                g_signal_new ("area_updated",
+                g_signal_new ("area-updated",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
                               G_STRUCT_OFFSET (GdkPixbufLoaderClass, area_updated),
@@ -208,10 +209,10 @@ gdk_pixbuf_loader_finalize (GObject *object)
  * Causes the image to be scaled while it is loaded. The desired
  * image size can be determined relative to the original size of
  * the image by calling gdk_pixbuf_loader_set_size() from a
- * signal handler for the ::size_prepared signal.
+ * signal handler for the ::size-prepared signal.
  *
  * Attempts to set the desired image size  are ignored after the 
- * emission of the ::size_prepared signal.
+ * emission of the ::size-prepared signal.
  *
  * Since: 2.2
  */
@@ -277,7 +278,15 @@ gdk_pixbuf_loader_prepare (GdkPixbuf          *pixbuf,
         else
                 anim = gdk_pixbuf_non_anim_new (pixbuf);
   
-        priv->animation = anim;
+	if (priv->needs_scale) {
+		priv->animation  = GDK_PIXBUF_ANIMATION (_gdk_pixbuf_scaled_anim_new (anim,
+                                         (double) priv->width / gdk_pixbuf_get_width (pixbuf),
+                                         (double) priv->height / gdk_pixbuf_get_height (pixbuf),
+					  1.0));
+			g_object_unref (anim);
+	}
+	else
+        	priv->animation = anim;
   
         if (!priv->needs_scale)
                 g_signal_emit (loader, pixbuf_loader_signals[AREA_PREPARED], 0);
@@ -473,7 +482,7 @@ gdk_pixbuf_loader_write (GdkPixbufLoader *loader,
  fail:
         gdk_pixbuf_loader_ensure_error (loader, error);
         gdk_pixbuf_loader_close (loader, NULL);
-        
+
         return FALSE;
 }
 
@@ -606,7 +615,7 @@ gdk_pixbuf_loader_new_with_mime_type (const char *mime_type,
  *
  * Queries the #GdkPixbuf that a pixbuf loader is currently creating.
  * In general it only makes sense to call this function after the
- * "area_prepared" signal has been emitted by the loader; this means
+ * "area-prepared" signal has been emitted by the loader; this means
  * that enough data has been read to know the size of the image that
  * will be allocated.  If the loader has not received enough data via
  * gdk_pixbuf_loader_write(), then this function returns %NULL.  The
@@ -640,9 +649,9 @@ gdk_pixbuf_loader_get_pixbuf (GdkPixbufLoader *loader)
  * @loader: A pixbuf loader
  *
  * Queries the #GdkPixbufAnimation that a pixbuf loader is currently creating.
- * In general it only makes sense to call this function after the "area_prepared"
+ * In general it only makes sense to call this function after the "area-prepared"
  * signal has been emitted by the loader. If the loader doesn't have enough
- * bytes yet (hasn't emitted the "area_prepared" signal) this function will 
+ * bytes yet (hasn't emitted the "area-prepared" signal) this function will 
  * return %NULL.
  *
  * Return value: The #GdkPixbufAnimation that the loader is loading, or %NULL if
@@ -711,9 +720,15 @@ gdk_pixbuf_loader_close (GdkPixbufLoader *loader,
 
         if (priv->image_module && priv->image_module->stop_load && priv->context) 
                 {
-                        if (!priv->image_module->stop_load (priv->context, error))
+                        GError *tmp = NULL;
+                        if (!priv->image_module->stop_load (priv->context, &tmp) || tmp)
                                 {
-                                        gdk_pixbuf_loader_ensure_error (loader, error);
+					/* don't call gdk_pixbuf_loader_ensure_error()
+ 					 * here, since we might not get an error in the
+ 					 * gdk_pixbuf_get_file_info() case
+ 					 */
+					if (tmp)
+						g_propagate_error (error, tmp);
                                         retval = FALSE;
                                 }
                 }
@@ -726,21 +741,8 @@ gdk_pixbuf_loader_close (GdkPixbufLoader *loader,
 
         if (priv->needs_scale) 
                 {
-                        GdkPixbuf *tmp, *pixbuf;
-                        
-                        tmp = gdk_pixbuf_animation_get_static_image (priv->animation);
-                        g_object_ref (tmp);
-                        pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, tmp->has_alpha, 8, priv->width, priv->height);
-                        g_object_unref (priv->animation);
-                        priv->animation = gdk_pixbuf_non_anim_new (pixbuf);
-                        g_object_unref (pixbuf);
+
                         g_signal_emit (loader, pixbuf_loader_signals[AREA_PREPARED], 0);
-                        gdk_pixbuf_scale (tmp, pixbuf, 0, 0, priv->width, priv->height, 0, 0,
-                                          (double) priv->width / tmp->width,
-                                          (double) priv->height / tmp->height,
-                                          GDK_INTERP_BILINEAR); 
-                        g_object_unref (tmp);
-                        
                         g_signal_emit (loader, pixbuf_loader_signals[AREA_UPDATED], 0, 
                                        0, 0, priv->width, priv->height);
                 }
