@@ -164,6 +164,8 @@ gtk_print_operation_init (GtkPrintOperation *operation)
   priv->unit = GTK_UNIT_PIXEL;
 
   appname = g_get_application_name ();
+  if (appname == NULL)
+    appname = "";
   /* translators: this string is the default job title for print
    * jobs. %s gets replaced by the application name, %d gets replaced
    * by the job number.
@@ -489,6 +491,12 @@ gtk_print_operation_preview_handler (GtkPrintOperation        *op,
 								  &dpi_x, &dpi_y,
 								  &pop->filename);
 
+  if (pop->surface == NULL)
+    {
+      g_free (pop);
+      return FALSE;
+    }
+
   cr = cairo_create (pop->surface);
   gtk_print_context_set_cairo_context (op->priv->print_context, cr,
 				       dpi_x, dpi_y);
@@ -667,8 +675,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    * Emitted for every page that is printed. The signal handler
    * must render the @page_nr's page onto the cairo context obtained
    * from @context using gtk_print_context_get_cairo_context().
-   *
-   * <informalexample><programlisting>
+   * |[
    * static void
    * draw_page (GtkPrintOperation *operation,
    *            GtkPrintContext   *context,
@@ -699,7 +706,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    *   pango_layout_set_width (layout, width * PANGO_SCALE);
    *   pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
    *      		      
-   *   pango_layout_get_size (layout, NULL, &amp;layout_height);
+   *   pango_layout_get_size (layout, NULL, &layout_height);
    *   text_height = (gdouble)layout_height / PANGO_SCALE;
    *   
    *   cairo_move_to (cr, width / 2,  (HEADER_HEIGHT - text_height) / 2);
@@ -707,7 +714,7 @@ gtk_print_operation_class_init (GtkPrintOperationClass *class)
    *   
    *   g_object_unref (layout);
    * }
-   * </programlisting></informalexample>
+   * ]|
    *
    * Use gtk_print_operation_set_use_full_page() and 
    * gtk_print_operation_set_unit() before starting the print operation
@@ -1842,9 +1849,9 @@ run_pdf (GtkPrintOperation  *op,
   if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS)
     {
       g_set_error (&priv->error,
-                   GTK_PRINT_ERROR,
-                   GTK_PRINT_ERROR_GENERAL,
-                   cairo_status_to_string (cairo_surface_status (surface)));
+		   GTK_PRINT_ERROR,
+		   GTK_PRINT_ERROR_GENERAL,
+		   cairo_status_to_string (cairo_surface_status (surface)));
       *do_print = FALSE;
       return GTK_PRINT_OPERATION_RESULT_ERROR;
     }
@@ -2270,13 +2277,38 @@ print_pages (GtkPrintOperation       *op,
 			     priv->print_context,
 			     parent,
 			     &handled);
-      
-      if (!handled ||
-	  gtk_print_context_get_cairo_context (priv->print_context) == NULL) 
-	{
-	  /* Programmer error */
-	  g_error ("You must set a cairo context on the print context");
-	}
+
+      if (!handled)
+        {
+          GtkWidget *error_dialog;
+
+          error_dialog = gtk_message_dialog_new (parent,
+                                                 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                 GTK_MESSAGE_ERROR,
+                                                 GTK_BUTTONS_OK,
+                                                 _("Error creating print preview"));
+
+          gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
+                                                    _("The most probable reason is that a temporary file could not be created."));
+
+          if (parent->group)
+            gtk_window_group_add_window (parent->group, GTK_WINDOW (error_dialog));
+
+          g_signal_connect (error_dialog, "response",
+                            G_CALLBACK (gtk_widget_destroy), NULL);
+
+          gtk_widget_show (error_dialog);
+
+          print_pages_idle_done (data);
+
+          return;
+        }
+
+      if (gtk_print_context_get_cairo_context (priv->print_context) == NULL)
+        {
+          /* Programmer error */
+          g_error ("You must set a cairo context on the print context");
+        }
       
       priv->start_page = preview_start_page;
       priv->end_page = preview_end_page;
@@ -2359,8 +2391,7 @@ gtk_print_operation_get_error (GtkPrintOperation  *op,
  * platform. The #GtkPrintOperation::done signal will be emitted with the 
  * operation results when the operation is done (i.e. when the dialog is 
  * canceled, or when the print succeeds or fails).
- *
- * <informalexample><programlisting>
+ * |[
  * if (settings != NULL)
  *   gtk_print_operation_set_print_settings (print, settings);
  *   
@@ -2368,11 +2399,14 @@ gtk_print_operation_get_error (GtkPrintOperation  *op,
  *   gtk_print_operation_set_default_page_setup (print, page_setup);
  *   
  * g_signal_connect (print, "begin-print", 
- *                   G_CALLBACK (begin_print), &amp;data);
+ *                   G_CALLBACK (begin_print), &data);
  * g_signal_connect (print, "draw-page", 
- *                   G_CALLBACK (draw_page), &amp;data);
+ *                   G_CALLBACK (draw_page), &data);
  *  
- * res = gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, parent, &amp;error);
+ * res = gtk_print_operation_run (print, 
+ *                                GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, 
+ *                                parent, 
+ *                                &error);
  *  
  * if (res == GTK_PRINT_OPERATION_RESULT_ERROR)
  *  {
@@ -2380,7 +2414,7 @@ gtk_print_operation_get_error (GtkPrintOperation  *op,
  *   			                     GTK_DIALOG_DESTROY_WITH_PARENT,
  * 					     GTK_MESSAGE_ERROR,
  * 					     GTK_BUTTONS_CLOSE,
- * 					     "Error printing file:\n&percnt;s",
+ * 					     "Error printing file:\n%s",
  * 					     error->message);
  *    g_signal_connect (error_dialog, "response", 
  *                      G_CALLBACK (gtk_widget_destroy), NULL);
@@ -2393,7 +2427,7 @@ gtk_print_operation_get_error (GtkPrintOperation  *op,
  *	g_object_unref (settings);
  *    settings = g_object_ref (gtk_print_operation_get_print_settings (print));
  *  }
- * </programlisting></informalexample>
+ * ]|
  *
  * Note that gtk_print_operation_run() can only be called once on a
  * given #GtkPrintOperation.
