@@ -24,13 +24,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 #include "gtkseparator.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
 
 
-G_DEFINE_TYPE (GtkSeparator, gtk_separator, GTK_TYPE_WIDGET);
+G_DEFINE_ABSTRACT_TYPE (GtkSeparator, gtk_separator, GTK_TYPE_WIDGET)
 
 static void
 gtk_separator_class_init (GtkSeparatorClass *class)
